/*
 * Copyright (C) 2002 TheUndying
 * Copyright (C) 2002 zap-zero
 * Copyright (C) 2002 Mihai RUSU (dizzy@rdsnet.ro)
 * Copyright (C) 2002 Zzzoom
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef WITH_MYSQL

#include "common/setup_before.h"
#include <stdio.h>

#ifdef STDC_HEADERS
# include <stdlib.h>
#else
# ifdef HAVE_MALLOC_H
#  include <malloc.h>
# endif
#endif

#ifdef HAVE_STRING_H
# include <string.h>
#else
# ifdef HAVE_STRINGS_H
#  include <strings.h>
# endif
#endif

#include "common/eventlog.h"
#include "prefs.h"
#include "common/util.h"
#include "account.h"
#include "pvpgn_mysql.h"
#include "common/setup_after.h"
#include "common/tag.h"

#define CURRENT_DB_VERSION 150

#define DB_MAX_ATTRKEY	128
#define DB_MAX_ATTRVAL  180
#define DB_MAX_TAB	64

static MYSQL *mysql = NULL;
static char * tables[] = {"BNET", "Record", "profile", "friend", "Team", NULL};

static int _dbcreator_init();
static int _dbcreator_createdatabase(char const *database);
static int _dbcreator_filldb(char const *database);

void update_DB_v0_to_v150(void);

static const char * _db_add_tab(const char *tab, const char *key)
{
   static char nkey[DB_MAX_ATTRKEY];
   char *p;
   int count;
   
   strncpy(nkey, tab, sizeof(nkey) - 1);
   nkey[strlen(nkey) + 1] = '\0'; nkey[strlen(nkey)] = '\\';
   strncpy(nkey + strlen(nkey), key, sizeof(nkey) - strlen(nkey));
   for(p = nkey; *p; p++)
     if (*p == '_') *p = '\\';
   
   return nkey;
}

static int _db_get_tab(const char *key, char **ptab, char **pcol)
{
   static char tab[DB_MAX_ATTRKEY];
   static char col[DB_MAX_ATTRKEY];
   char *p;
   
   strncpy(tab, key, DB_MAX_TAB-1);
   tab[DB_MAX_TAB-1]=0;

   if(!strchr(tab, '\\')) return -1;

   
   *(strchr(tab, '\\')) = 0;
   strncpy(col, key+strlen(tab)+1, DB_MAX_TAB-1);
   col[DB_MAX_TAB-1]=0;
   for(p=col; *p; p++)
     if(*p == '\\' || *p == ' ')
       *p = '_';
   
   /* return tab and col as 2 static buffers */
   *ptab = tab;
   *pcol = col;
   return 0;
}

extern int db_init()
{
   if (prefs_get_mysql_persistent() && mysql != NULL) {
      if (mysql_ping(mysql)) {
         eventlog(eventlog_level_error, "db_init", "lost connection to database");
         return -1;
      }
      return 0;
   }

   if (!((mysql=mysql_init(NULL)))) {
      eventlog(eventlog_level_error, "db_init", "mysql init failed");
      return -1; 
   }
   
   eventlog(eventlog_level_trace, "db_init", "host: '%s', account: '%s', sock: '%s'", prefs_get_mysql_host(), prefs_get_mysql_account(), prefs_get_mysql_sock());
   
   if(!mysql_real_connect(mysql, prefs_get_mysql_host(), prefs_get_mysql_account(), prefs_get_mysql_password(), prefs_get_mysql_dbname(), 0, prefs_get_mysql_sock(), CLIENT_FOUND_ROWS)) 
     {
	eventlog(eventlog_level_error, "db_init", "mysql connection failed (%s)",mysql_error(mysql));
	db_close();
	return -1;
     }
   
   return 0;
}

extern int db_close()
{
	if (!mysql) 
	{
		eventlog(eventlog_level_error, "db_close", "mysql init failed");
		return -1; 
	}

	mysql_close(mysql);
	mysql = NULL;
	return 0;
}

int db_get_version()
{
  MYSQL_RES * result = NULL;
  MYSQL_ROW row;
  int version = 0;
	
  db_init();
  if (!mysql) return -1;
  mysql_query(mysql,"SELECT version FROM Version");
  result = mysql_store_result(mysql);
  if (!result) return 0;
  if (mysql_num_rows(result) != 1) { mysql_free_result(result); return 0; }
  if (!(row = mysql_fetch_row(result))) { mysql_free_result(result); return 0; }
  if (row[0]==NULL) { mysql_free_result(result); return 0; }
  version = atoi(row[0]);
  mysql_free_result(result);
  db_close();
  return version;
}

void db_set_version(int version)
{
  char query[1024];
       
  db_init();
  if (!mysql) return;
  
  sprintf(query,"UPDATE Version SET version=%d;",version);
  eventlog(eventlog_level_info,__FUNCTION__,"query:%s",query);
  mysql_query(mysql,query);
  if (mysql_affected_rows(mysql)!=1) 
    {
      mysql_query(mysql,"CREATE TABLE Version (version int(11) NOT NULL PRIMARY KEY);");
      sprintf(query,"INSERT INTO Version VALUES (%d);",version);
      mysql_query(mysql,query);
    }
  db_close();
}

extern int db_check()
{
  int version = 0;
   if (prefs_get_mysql_host() == NULL || prefs_get_mysql_dbname() == NULL ||
       prefs_get_mysql_account() == NULL || prefs_get_mysql_password() == NULL) {
      eventlog(eventlog_level_error, "db_check", "Invalid mysql settings! Please check your config!");
      return -1;
   }
   
   /* settings seem fine, lets try to connect */
   if ((mysql = mysql_init(NULL)) == NULL) {
      eventlog(eventlog_level_error, "db_check", "Could not init mysql");
      return -1;
   }
   
   if(!mysql_real_connect(mysql, prefs_get_mysql_host(), prefs_get_mysql_account(), prefs_get_mysql_password(), prefs_get_mysql_dbname(), 0, prefs_get_mysql_sock(), CLIENT_FOUND_ROWS)) 
     {
	eventlog(eventlog_level_info, "db_check", "mysql connection failed, lets try to build the db (%s)",mysql_error(mysql));
	db_close(); /* dbcreator opens its own connection */
	
	if (dbcreator_start() <0) {
	   eventlog(eventlog_level_error,"db_check","could not build db");
	   return -1;
	}

	return 0;
     }
   
   db_close();
   
   while ((version=db_get_version())!=CURRENT_DB_VERSION)
     {
       switch (version)
	 {
	 case 0:
	   {
             update_DB_v0_to_v150();
	     break;
	   }
	 default:
	   {
	     eventlog(eventlog_level_error,__FUNCTION__,"unknown PvPGN DB version, aborting");
	     return -1;
	     
	   }
	 }
     }
   
   return 0;
}

static unsigned int db_max_uid()
{
   MYSQL_RES* result = NULL;
   MYSQL_ROW row;
   int uid;
   
   if(!mysql) {
      eventlog(eventlog_level_error, "db_max_uid", "NULL mysql");
      return 0;
   }
   
   mysql_query(mysql,"SELECT max_uid FROM counters;");
   result = mysql_store_result(mysql);
   if(result) {
      row = mysql_fetch_row(result);
      /* Dizzy: 16-07-2002 : check to see if its NULL in db */
      if (row == NULL || row[0] == NULL) {
	 eventlog(eventlog_level_error, "db_max_uid", "got NULL maxuid");
	 mysql_free_result(result);
	 return 0;
      }
      uid = atol(row[0]);
      mysql_free_result(result);
   }
   else return 0;
   
   eventlog(eventlog_level_trace, "db_max_uid", "uid: %d", uid);
   
   return uid;
}

extern int db_new_user(char const * name)
{
	char query[1024];
        MYSQL_RES* result = NULL;
	MYSQL_ROW row;
	int aff_rows = 0;
	int uid = db_max_uid() + 1;
	char str_uid[32];

        if(!mysql) {
		eventlog(eventlog_level_error, "db_new_user", "NULL mysql");
		return 0;
	}

	sprintf(str_uid, "%u", uid);
	
	/* Dizzy:  Jul 15 2002 */
	/* db_new_user did not checked for an existant account with the same name */
	sprintf(query, "SELECT count(*) FROM BNET WHERE acct_username='%s'", name);
	mysql_query(mysql, query);
        result = mysql_store_result(mysql);
        if(result) {
            int num;

            row = mysql_fetch_row(result);
	    if (row == NULL || row[0] == NULL) {
		mysql_free_result(result);
		eventlog(eventlog_level_error, "db_new_user", "got NULL count");
		return 0;
	    }
            num = atol(row[0]);
            mysql_free_result(result);
            if (num>0) {
		eventlog(eventlog_level_error, "db_new_user", "got existant username");
		return 0;
	    }
        }
        else {
	    eventlog(eventlog_level_error, "db_new_user", "error trying query: \"%s\"", query);
	    return 0;
	}

	sprintf(query, "UPDATE counters SET max_uid='%s';", str_uid);
        mysql_query(mysql, query);

	eventlog(eventlog_level_trace, "db_new_user", "query: %s", query);

	if(mysql_affected_rows(mysql) != 1) {
	        eventlog(eventlog_level_error, "db_new_user", "uid update failed");
		return 0;
	}

	sprintf(query, "INSERT INTO BNET (uid) VALUES('%s');", str_uid);
	mysql_query(mysql,query); aff_rows += mysql_affected_rows(mysql);
	sprintf(query, "INSERT INTO profile (uid) VALUES('%s');", str_uid);
	mysql_query(mysql,query); aff_rows += mysql_affected_rows(mysql);
	sprintf(query, "INSERT INTO Record (uid) VALUES('%s');", str_uid);
	mysql_query(mysql,query); aff_rows += mysql_affected_rows(mysql);
	sprintf(query, "INSERT INTO friend (uid) VALUES('%s');", str_uid);
	mysql_query(mysql,query); aff_rows += mysql_affected_rows(mysql);

	// throw away result
        result = mysql_store_result(mysql);
        if(result)
	        mysql_free_result(result);

	if(aff_rows != 4) {
	        eventlog(eventlog_level_error, "db_new_user", "user insert failed");
		return 0;
	}

	return uid;
}

t_uidlist * db_get_accounts(void)
{
   char query[1024];
   MYSQL_RES* result = NULL;
   MYSQL_ROW row;
   t_uidlist *list;
   
   if(!mysql) {
      eventlog(eventlog_level_error, "db_get_accounts", "NULL mysql");
      return NULL;
   }
   
   strcpy(query,"SELECT DISTINCT uid FROM BNET");
   mysql_query(mysql, query);
   result = mysql_store_result(mysql);

   if(result && (mysql_num_rows(result) > 1)) {
      int norows;
      unsigned int num, count;
      
      norows = mysql_num_rows(result) - 1;
      if ((list = malloc(sizeof(t_uidlist))) == NULL) {
	 eventlog(eventlog_level_error, "db_get_accounts","could not allocate for list");
	 mysql_free_result(result);
	 return NULL;
      }
      
      if ((list->uids = malloc(sizeof(unsigned int) * norows)) == NULL) {
	 eventlog(eventlog_level_error, "db_get_accounts","could not allocate for uids");
	 free(list);
	 mysql_free_result(result);
	 return NULL;
      }
      
      list->len = norows;

      count = 0;
      for(count = 0; ((row = mysql_fetch_row(result))) && count < norows; count++) {
	 if (row[0] == NULL) {
	    free(list->uids);
	    free(list);
	    mysql_free_result(result);
	    eventlog(eventlog_level_error, "db_get_accounts", "got NULL uid");
	    return NULL;
	 }
	 num = atol(row[0]);
	 
	 if (num == 0) {count--; continue;}
	 list->uids[count] = num;
      }

      mysql_free_result(result);
   } else {
      eventlog(eventlog_level_error,"db_get_accounts","error query db (query:\"%s\")", query);
      if (result) mysql_free_result(result);
      return NULL;
   }
   return list;
}

void db_free_accounts(t_uidlist *list)
{
   if (list && list->len > 0) {
      if (list->uids) free(list->uids);
      free(list);
   }
}

t_attrlist * db_get_attributes(unsigned int uid)
{
   char query[1024];
   MYSQL_RES* result = NULL;
   MYSQL_ROW row;
   t_attrlist *list;
   char **tab;
   int maxlen;

   eventlog(eventlog_level_trace, "db_get_attributes", "<<< ENTER! (uid: %u)",uid);
   if(!mysql) {
      eventlog(eventlog_level_error, "db_get_attributes", "NULL mysql");
      return NULL;
   }
   
   
   if ((list = malloc(sizeof(t_attrlist))) == NULL) {
      eventlog(eventlog_level_error,"db_get_attributes","could not allocate list");
      return NULL;
   }
   
   list->len = 0; maxlen = 0;
   list->keys = NULL; list->values = NULL;

   for(tab = tables; *tab ; tab++) {

      sprintf(query,"SELECT * FROM %s WHERE uid='%u'", *tab, uid);
      mysql_query(mysql, query);
      result = mysql_store_result(mysql);

      eventlog(eventlog_level_trace,"db_get_attributes", "query: \"%s\"",query);

      if (result && mysql_num_rows(result) == 1 && mysql_num_fields(result)>1) {
	 int nofields, i;
	 MYSQL_FIELD *fields;
	 char **nkeys, **nvalues;
	 
	 if ((fields = mysql_fetch_fields(result)) == NULL) {
	    eventlog(eventlog_level_error,"db_get_attributes","could not fetch the fields");
	    mysql_free_result(result);
	    db_free_attributes(list);
	    return NULL;
	 }
	 nofields = mysql_num_fields(result) - 1;

	 if (!(row = mysql_fetch_row(result))) {
	    eventlog(eventlog_level_error,"db_get_attributes","could not fetch row");
	    mysql_free_result(result);
	    db_free_attributes(list);
	    return NULL;
	 }
	 
	 if ((nkeys = malloc(sizeof(char*) * (list->len + nofields))) == NULL) {
	    eventlog(eventlog_level_error,"db_get_attributes","could not extend attributes (keys)");
	    mysql_free_result(result);
	    db_free_attributes(list);
	    return NULL;
	 }
	 
	 if ((nvalues = malloc(sizeof(char*) * (list->len + nofields))) == NULL) {
	    eventlog(eventlog_level_error,"db_get_attributes","could not extend attributes (values)");
	    mysql_free_result(result);
	    db_free_attributes(list);
	    free(nkeys);
	    return NULL;
	 }
	 
	 /* lets init all pointers to NULL */
	 memset(nkeys, 0, sizeof(char *) * (list->len + nofields));             
    	 memset(nvalues, 0, sizeof(char *) * (list->len + nofields));
	 
	 if (list->len) {
	    memmove(nkeys, list->keys, sizeof(char*) * list->len);
	    memmove(nvalues, list->values, sizeof(char*) * list->len);
	    free(list->keys);
	    free(list->values);
	 }

	 list->keys = nkeys;
	 list->values = nvalues;
	 maxlen += nofields;

	 for(i=0; i<=nofields; i++) { /* we have to skip "uid" */
	    if (fields[i].name == NULL) {
	       eventlog(eventlog_level_error,"db_get_attributes","could not read a field name");
	       mysql_free_result(result);
	       db_free_attributes(list);
	       return NULL;
	    }
	    
	    /* we ignore the field used internally by sql */
	    if (strcmp(fields[i].name, MYSQL_UID_FIELD) == 0) continue;

	    if (row[i] == NULL) continue; /* its an uninitialized attribute */

	    if ((list->keys[list->len] = strdup(_db_add_tab(*tab, fields[i].name))) == NULL) {
	       eventlog(eventlog_level_error,"db_get_attributes","could not duplicate key");
	       mysql_free_result(result);
	       db_free_attributes(list);
	       return NULL;
	    }

	    if ((list->values[list->len] = (unescape_chars(row[i]))) == NULL) {
	       eventlog(eventlog_level_error,"db_get_attributes","could not duplicate value");
	       free(list->keys[list->len]); list->keys[list->len] = NULL;
	       mysql_free_result(result);
	       db_free_attributes(list);
	       return NULL;
	    }
	    
/*	    eventlog(eventlog_level_trace,"db_get_attributes","read: \"%s\" \"%s\"", list->keys[list->len], list->values[list->len]); */
	    list->len++;
	    /* sanity check, this should not happen */
	    if (list->len >= maxlen) {
		eventlog(eventlog_level_error,"db_get_attributes","attribute overflow, no 'uid' field ?");
		break;
	    }
	 }
      }
      mysql_free_result(result);
   }
   
   return list;
}

void db_free_attributes(t_attrlist *list)
{
   eventlog(eventlog_level_trace,"db_free_attributes","<<<< ENTER!");
   if (list && list->len) {
      int i;
      
      if (list->keys) {
	 for(i=0; i<list->len; i++)
	   if (list->keys[i]) free(list->keys[i]);
	 free(list->keys);
      }

      if (list->values) {
	 for(i=0; i<list->len; i++)
	   if (list->values[i]) free(list->values[i]);
	 free(list->values);
      }
      
      free(list);
   }
}

int db_set(unsigned int userid, const char *key, const char *val)
{
   char query[1024];
   char escape[DB_MAX_ATTRVAL * 2 + 1]; /* mysql docs say the escape can take a maximum of double original size */
   char safeval[DB_MAX_ATTRVAL];
   char *p, *tab, *col;
   
   if(!mysql) {
      eventlog(eventlog_level_error, "db_set", "NULL mysql");
      return -1;
   }

   if (_db_get_tab(key, &tab, &col)<0) {
      eventlog(eventlog_level_error,"db_set","error from db_get_tab");
      return -1;
   }

   strncpy(safeval, val, DB_MAX_ATTRVAL-1);
   safeval[DB_MAX_ATTRVAL-1]=0;
   for(p=safeval; *p; p++)
     if(*p == '\'')              /* value shouldn't contain ' */
       *p = '"';
   
   mysql_real_escape_string(mysql, escape, safeval, strlen(safeval));
   
   strcpy(query, "UPDATE ");
   strncat(query,tab,64);
   strcat(query," SET ");
   strncat(query,col,64);
   strcat(query,"='");
   strcat(query,escape);
   strcat(query,"' WHERE uid='");
   sprintf(query + strlen(query), "%u", userid);
   strcat(query,"';");
   
   eventlog(eventlog_level_trace, "db_set", "query: %s", query);
   mysql_query(mysql,query);
   
   if(mysql_affected_rows(mysql) != 1) {
      char query2[512];
      eventlog(eventlog_level_info, "db_set", "trying to insert new column %s", col);
      strcpy(query2, "ALTER TABLE ");
      strncat(query2,tab,DB_MAX_TAB);
      strcat(query2," ADD COLUMN ");
      strncat(query2,col,DB_MAX_TAB);
      strcat(query2," VARCHAR(128);");

      eventlog(eventlog_level_trace, "db_set", "query: %s", query2);
      if (mysql_query(mysql,query2)) {
	    eventlog(eventlog_level_error,"db_set","ALTER failed, bailing out");
	    return -1;
      }

      /* try query again */
      eventlog(eventlog_level_trace, "db_set", "query: %s", query);
      if (mysql_query(mysql,query) || mysql_affected_rows(mysql) != 1) {
        // Added by QWERTY
        // Tried everything, now trying to insert that user to the table for the first time
        sprintf(query2,"INSERT INTO %s (uid,%s) VALUES ('%u','%s')",tab,col,userid,escape);
        eventlog(eventlog_level_error, "db_set", "update failed so tried INSERT for the last chance");
        if (mysql_query(mysql,query2) || mysql_affected_rows(mysql)!=1) {
                eventlog(eventlog_level_error, "db_set", "Last chance report: NEGATIVE");
		return -1;
        }
        eventlog(eventlog_level_error, "db_set", "Last chance report: POSITIVE");
        return 0;
      }
   }

   return 0;
}

extern char const * db_get_attribute(unsigned int sid, char const * key)
{
   char query[1024];
   char *p, *tab, *col;
   MYSQL_RES* result = NULL;
   MYSQL_ROW row;
   char const * val;
   
   if(!mysql) {
      eventlog(eventlog_level_error, "db_get_attribute", "NULL mysql");
      return NULL;
   }

   if (_db_get_tab(key, &tab, &col)<0) {
      eventlog(eventlog_level_error,"db_get_attribute","error from db_get_tab");
      return NULL;
   }
   
   sprintf(query, "SELECT %s FROM %s WHERE uid = '%u';",col,tab,sid);
   
   eventlog(eventlog_level_trace, "db_get_attribute", "query: %s", query);
   mysql_query(mysql,query);

   result = mysql_store_result(mysql);
   
   if (!result) return NULL;
   if (mysql_num_rows(result) != 1) { mysql_free_result(result); return NULL; }
   
   if (!(row = mysql_fetch_row(result))) {
     eventlog(eventlog_level_error,"db_get_attribute","could not fetch row");
     mysql_free_result(result);
     return NULL;
   }
   
   if (row[0]==NULL)
   {
     mysql_free_result(result);
     return NULL;	
   }

   if ((val = (unescape_chars(row[0]))) == NULL) {
     eventlog(eventlog_level_error,"db_get_attribute","could not duplicate value");
     mysql_free_result(result);
     return NULL;
   }

   mysql_free_result(result);
   
   return val;
   


}

// aaron ---->
extern int db_drop_column(const char * key)
{
  char query[1024];
  char *tab, *col;
  
  if(!mysql) {
    eventlog(eventlog_level_error, "db_set", "NULL mysql");
    return -1;
  }

  if (_db_get_tab(key, &tab, &col)<0) {
    eventlog(eventlog_level_error,"db_set","error from db_get_tab");
    return -1;
  }
   
  strcpy(query, "ALTER TABLE ");
  strncat(query,tab,64);
  strcat(query," DROP ");
  strncat(query,col,64);
  strcat(query,"';");
  
  eventlog(eventlog_level_trace, "db_drop_column", "query: %s", query);
  mysql_query(mysql,query);
  
  if(mysql_affected_rows(mysql) != 1) 
    {
      eventlog(eventlog_level_fatal,"db_drop_column","Failed to drop column \"%s\" on table \"%s\".",col,tab);
      return -1;
    }  
  
  
  return 0;
}
// <---


extern int dbcreator_start()
{
   if(_dbcreator_init()<0)
     {
	eventlog(eventlog_level_error,"dbcreator_start","Error connection to MySQL - Check your conf File!");
	return -1;
     }
   
   eventlog(eventlog_level_info,"pvpgn_mysql","Creating Database %s",prefs_get_mysql_dbname());
   
   if(_dbcreator_createdatabase(prefs_get_mysql_dbname())<0) {
      eventlog(eventlog_level_error, "dbcreator_start","Error trying to create the database: %s", prefs_get_mysql_dbname());
      db_close();
      return -1;	
   }
   if(_dbcreator_filldb(prefs_get_mysql_dbname())<0) {
      eventlog(eventlog_level_error, "dbcreator_start", "Error trying to init db struct");
      db_close();
      return -1;
   }
   
   db_close();

   return 0;
   
}

static int _dbcreator_init()
{
   if (!((mysql=mysql_init(NULL)))) 
     return -1; 
   
   if(!mysql_real_connect(mysql, prefs_get_mysql_host(), prefs_get_mysql_account(), prefs_get_mysql_password(), "mysql", 0, prefs_get_mysql_sock(), CLIENT_FOUND_ROWS))
   {
     eventlog(eventlog_level_error,__FUNCTION__,"error: %s",mysql_error(mysql));
     return -1;
   }

   return 0;
}

static int _dbcreator_createdatabase(char const *database)
{
	char query[1024];
	
	strcpy(query,"CREATE DATABASE ");
	strncat(query,database, sizeof(query) - strlen(query) - 2);
	strcat(query,";");
	
	mysql_query(mysql,query);

	if(mysql_affected_rows(mysql) != 1) 
	{
		eventlog(eventlog_level_fatal,"dbcreator_createdatabase","Failed to Create the database.");
		return -1;
	}

   eventlog(eventlog_level_info,"dbcreator_createdatabase","Database Created...creating Tables..");
   
   return 0;
}

static int _dbcreator_filldb(char const *database)
{
   char **pstr;
   char * querys[] = {
      "CREATE TABLE BNET (uid int(11) NOT NULL default '0' PRIMARY KEY,acct_username varchar(128) default NULL,acct_userid varchar(128) default NULL,acct_passhash1 varchar(128) default NULL,flags_initial varchar(128) default NULL,auth_admin varchar(128) default 'false',auth_normallogin varchar(128) default NULL,auth_changepass varchar(128) default NULL,auth_changeprofile varchar(128) default NULL,auth_botlogin varchar(128) default 'true',auth_operator varchar(128) default NULL,new_at_team_flag varchar(128) default '0', auth_lockk varchar(128) default '0', auth_command_groups varchar(128) NOT NULL default '1');",
      "INSERT INTO BNET VALUES (0,NULL,NULL,NULL,NULL,'false',NULL,NULL,NULL,'true',NULL,'0','0','1');",
      "CREATE TABLE counters (max_uid int(11) NOT NULL default '0' PRIMARY KEY);",
      "INSERT INTO counters VALUES (0);",
      "CREATE TABLE friend (uid int(11) NOT NULL default '0' PRIMARY KEY);",
      "INSERT INTO friend VALUES (0);",
      "CREATE TABLE profile (uid int(11) NOT NULL default '0' PRIMARY KEY);",
      "INSERT INTO profile VALUES (0);",
      "CREATE TABLE Record (uid int(11) NOT NULL default '0' PRIMARY KEY, WAR3_solo_xp int(11) default '0', WAR3_solo_level int(11) default '0', WAR3_solo_wins int(11) default '0', WAR3_solo_rank int(11) default '0', WAR3_solo_losses int(11) default '0', WAR3_team_xp int(11) default '0', WAR3_team_level int(11) default '0', WAR3_team_rank int(11) default '0', WAR3_team_wins int(11) default '0', WAR3_team_losses int(11) default '0', WAR3_ffa_xp int(11) default '0', WAR3_ffa_rank int(11) default '0', WAR3_ffa_level int(11) default '0', WAR3_ffa_wins int(11) default '0', WAR3_ffa_losses int(11) default '0', WAR3_orcs_wins int(11) default '0', WAR3_orcs_losses int(11) default '0', WAR3_humans_wins int(11) default '0', WAR3_humans_losses int(11) default '0', WAR3_undead_wins int(11) default '0', WAR3_undead_losses int(11) default '0', WAR3_nightelves_wins int(11) default '0', WAR3_nightelves_losses int(11) default '0', WAR3_random_wins int(11) default '0', WAR3_random_losses int(11) default '0', WAR3_teamcount int(11) default '0', W3XP_solo_xp int(11) default '0', W3XP_solo_level int(11) default '0', W3XP_solo_wins int(11) default '0', W3XP_solo_rank int(11) default '0', W3XP_solo_losses int(11) default '0', W3XP_team_xp int(11) default '0', W3XP_team_level int(11) default '0', W3XP_team_rank int(11) default '0', W3XP_team_wins int(11) default '0', W3XP_team_losses int(11) default '0', W3XP_ffa_xp int(11) default '0', W3XP_ffa_rank int(11) default '0', W3XP_ffa_level int(11) default '0', W3XP_ffa_wins int(11) default '0', W3XP_ffa_losses int(11) default '0', W3XP_orcs_wins int(11) default '0', W3XP_orcs_losses int(11) default '0', W3XP_humans_wins int(11) default '0', W3XP_humans_losses int(11) default '0', W3XP_undead_wins int(11) default '0', W3XP_undead_losses int(11) default '0', W3XP_nightelves_wins int(11) default '0', W3XP_nightelves_losses int(11) default '0', W3XP_random_wins int(11) default '0', W3XP_random_losses int(11) default '0', W3XP_teamcount int(11) default '0');",
      "INSERT INTO Record VALUES (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);",
	  "CREATE TABLE Team (uid int(11) NOT NULL PRIMARY KEY);",
	  "INSERT INTO Team VALUES (0);",
	  "CREATE TABLE Version (version int(11) NOT NULL PRIMARY KEY);",
	  "INSERT INTO Version VALUES(150);",
      NULL
   };
	              
   if (mysql_select_db(mysql, database)) return -1;
   
   for(pstr = querys; *pstr; pstr++)
     if (mysql_query(mysql, *pstr)) return -1;
    
    return 0;
}

void update_DB_v0_to_v150()
{
  MYSQL_RES   * result;
  MYSQL_FIELD * field;
  char query[1024];
  int i;
  
  db_init();

  if (!mysql) return;
  
  eventlog(eventlog_level_info,__FUNCTION__,"updating your PvPGN mySQL DB...");

  mysql_query(mysql,"SELECT * FROM Record;");
  if ((result = mysql_store_result(mysql)))
  {
    while ((field = mysql_fetch_field(result)))
    {
      if (field->name==NULL) continue;
      if (strncmp(field->name,"WAR3_",5)==0) continue;   // prevent converting over and over again
      if (strncmp(field->name,"W3XP_",5)==0) continue;
      if (strncmp(field->name,CLIENTTAG_STARCRAFT,4)==0) continue;
      if (strncmp(field->name,CLIENTTAG_BROODWARS,4)==0) continue;
      if (strncmp(field->name,CLIENTTAG_WARCIIBNE,4)==0) continue;
      if (strncmp(field->name,CLIENTTAG_DIABLO2DV,4)==0) continue;
      if (strncmp(field->name,CLIENTTAG_DIABLO2XP,4)==0) continue;
      if (strncmp(field->name,CLIENTTAG_DIABLORTL,4)==0) continue;
      if (strcmp(field->name,MYSQL_UID_FIELD)==0) continue;
      if (strcmp(field->name,"uid")==0) continue;

      sprintf(query,"ALTER TABLE Record CHANGE %s WAR3_%s int(11) default '0';",field->name,field->name);
      mysql_query(mysql,query);
      sprintf(query,"ALTER TABLE Record ADD W3XP_%s int(11) default '0';",field->name);
      mysql_query(mysql,query); 
    }
   mysql_free_result(result);
  }

  mysql_query(mysql,"SELECT * FROM Team;");
  if ((result = mysql_store_result(mysql)))
  {
    while ((field = mysql_fetch_field(result)))
    {
      if (field->name==NULL) continue;
      if (strncmp(field->name,"WAR3_",5)==0) continue;
      if (strncmp(field->name,"W3XP_",5)==0) continue;
      if (strcmp(field->name,MYSQL_UID_FIELD)==0) continue;
      if (strcmp(field->name,"uid")==0) continue;

      sprintf(query,"ALTER TABLE Team CHANGE %s WAR3_%s varchar(128);",field->name,field->name);
      mysql_query(mysql,query);
    }
    mysql_free_result(result);
  }

  db_close();
  db_set_version(150);

  eventlog(eventlog_level_info,__FUNCTION__,"successfully updated your DB");
}

#endif

